@@ -19,6 +19,9 @@
 import functools
 import weakref
 from contextlib import contextmanager
+from typing import Generator, Iterator
+from itertools import compress, repeat
+from inspect import signature
 
 import torch
 
@@ -197,7 +200,7 @@ def zero_wrap(zero_params):
 
         Parameters
         ----------
-        func: function
+        modifier: function
             Function to wrap.
 
         Returns
@@ -234,6 +237,44 @@ def zero_wrap(zero_params):
     return zero_params_wrapper
 
 
+def uncompress(data, selector, compressed) -> Generator:
+    '''Generator which, given a compressed iterable produced by :py:obj:`itertools.compress` and (some iterable similar
+    to) the original data and selector used for :py:obj:`~itertools.compress`, yields values from `compressed` or
+    `data` depending on `selector`. `True` values in `selector` skip `data` one ahead and yield a value from
+    `compressed`, while `False` values yield one value from `data`.
+
+    Parameters
+    ----------
+    data : iterable
+        The iterable (similar to the) original data. `False` values in the `selector` will be filled with values from
+        this iterator, while `True` values will cause this iterable to be skipped.
+    selector : iterable of bool
+        The original selector used to produce `compressed`. Chooses whether elements from `data` or from `compressed`
+        will be yielded.
+    compressed : iterable
+        The results of :py:obj:`itertools.compress`. Will be yielded for each `True` element in `selector`.
+
+    Yields
+    ------
+    object
+        An element of `data` if the associated element of `selector` is `False`, otherwise an element of `compressed`
+        while skipping `data` one ahead.
+
+    '''
+    its = iter(selector)
+    itc = iter(compressed)
+    itd = iter(data)
+    for select in its:
+        try:
+            if select:
+                next(itd)
+                yield next(itc)
+            else:
+                yield next(itd)
+        except StopIteration:
+            break
+
+
 class ParamMod:
     '''Class to produce a context manager to temporarily modify parameter attributes (all by default) of a module.
 
@@ -256,6 +297,51 @@ class ParamMod:
         self.zero_params = zero_params
         self.require_params = require_params
 
+    def state_dicts(self, module):
+        '''Returns a state_dict of the modified module parameters.
+
+        Parameters
+        ----------
+        module: :py:obj:`torch.nn.Module`
+            The module for which parameters shall be modified.
+
+        Returns
+        -------
+        original_state: dict of :py:obj:`torch.Tensor`
+            The original, unmodified parameters.
+        modified_state: dict of :py:obj:`torch.Tensor`
+            The modified parameters.
+
+        Raises
+        ------
+        RuntimeError
+            If parameters are missing and `self.require_params` has been set to ``True``.
+        '''
+        param_keys = self.param_keys
+        zero_params = self.zero_params
+
+        if param_keys is None:
+            param_keys = [name for name, _ in module.named_parameters(recurse=False)]
+        if zero_params is None:
+            zero_params = []
+
+        missing = [key for key in param_keys if not hasattr(module, key)]
+        if self.require_params and missing:
+            missing_str = '\', \''.join(missing)
+            raise RuntimeError(f'Module {module} requires missing parameters: \'{missing_str}\'')
+
+        modifier = zero_wrap(zero_params)(self.modifier)
+
+        modified_state = {}
+        original_state = {}
+        for key in param_keys:
+            if key not in missing:
+                param = getattr(module, key)
+                if param is not None:
+                    original_state[key] = param.data
+                    modified_state[key] = modifier(param.data, key)
+        return original_state, modified_state
+
     @classmethod
     def ensure(cls, modifier):
         '''If ``modifier`` is an instance of ParamMod, return it as-is, if it is callable, create a new instance with
@@ -283,7 +369,7 @@ class ParamMod:
         raise TypeError(f'{modifier} is neither an instance of {cls}, nor callable!')
 
     @contextmanager
-    def __call__(self, module):
+    def __call__(self, module) -> Generator:
         '''Context manager to temporarily modify parameter attributes (all by default) of a module.
 
         Parameters
@@ -292,47 +378,22 @@ class ParamMod:
             Module of which to modify parameters. If `self.requires_params` is `True`, it must have all elements given
             in `self.param_keys` as attributes (attributes are allowed to be `None`, in which case they are ignored).
 
-        Raises
-        ------
-        RuntimeError
-            If `self.require_params` is `True` and `module` is missing an attribute listed in `self.param_keys`.
-
         Yields
         ------
         module: :py:obj:`torch.nn.Module`
             The `module` with appropriate parameters temporarily modified.
         '''
+        # assign empty dict, as either the following two functions may crash
+        original_state = {}
         try:
-            stored_params = {}
-            param_keys = self.param_keys
-            zero_params = self.zero_params
-
-            if param_keys is None:
-                param_keys = [name for name, _ in module.named_parameters(recurse=False)]
-            if zero_params is None:
-                zero_params = []
-
-            missing = [key for key in param_keys if not hasattr(module, key)]
-            if self.require_params and missing:
-                missing_str = '\', \''.join(missing)
-                raise RuntimeError(f'Module {module} requires missing parameters: \'{missing_str}\'')
-
-            modifier = zero_wrap(zero_params)(self.modifier)
-
-            for key in param_keys:
-                if key not in missing:
-                    param = getattr(module, key)
-                    if param is not None:
-                        stored_params[key] = param
-                        setattr(module, key, torch.nn.Parameter(modifier(param.data, key)))
-
+            original_state, modified_state = self.state_dicts(module)
+            module.load_state_dict(modified_state, strict=False, assign=True)
             yield module
         finally:
-            for key, value in stored_params.items():
-                setattr(module, key, value)
+            module.load_state_dict(original_state, strict=False, assign=True)
 
 
-def collect_leaves(module):
+def collect_leaves(module) -> Iterator[torch.nn.Module]:
     '''Generator function to collect all leaf modules of a module.
 
     Parameters
@@ -350,8 +411,7 @@ def collect_leaves(module):
     children = module.children()
     for child in children:
         is_leaf = False
-        for leaf in collect_leaves(child):
-            yield leaf
+        yield from collect_leaves(child)
     if is_leaf:  # pragma: no branch
         yield module
 
@@ -360,12 +420,39 @@ class Identity(torch.autograd.Function):
     '''Identity to add a grad_fn to a tensor, so a backward hook can be applied.'''
     @staticmethod
     def forward(ctx, *inputs):
-        '''Forward identity.'''
+        '''Forward identity.
+
+        Parameters
+        ----------
+        ctx: object
+            The function context.
+        *inputs: tuple of :py:obj:`torch.Tensor`
+            Inputs to forward.
+
+        Returns
+        -------
+        inputs: tuple of :py:obj:`torch.Tensor`
+            The unmodified inputs.
+        '''
+        ctx.mark_non_differentiable(*[elem for elem in inputs if not elem.requires_grad])
         return inputs
 
     @staticmethod
     def backward(ctx, *grad_outputs):
-        '''Backward identity.'''
+        '''Backward identity.
+
+        Parameters
+        ----------
+        ctx: object
+            The function context.
+        *grad_outputs: tuple of :py:obj:`torch.Tensor`
+            Output gradients.
+
+        Returns
+        -------
+        grad_outputs: tuple of :py:obj:`torch.Tensor`
+            The unmodified output gradients.
+        '''
         return grad_outputs
 
 
@@ -376,85 +463,226 @@ class Hook:
         self.active = True
         self.tensor_handles = RemovableHandleList()
 
-    def pre_forward(self, module, input):
-        '''Apply an Identity to the input before the module to register a backward hook.'''
+    @staticmethod
+    def _inject_grad_fn(args):
+        tensor_mask = tuple(isinstance(elem, torch.Tensor) for elem in args)
+        tensors = tuple(compress(args, tensor_mask))
+        # tensors = [(n, elem) for elem in enumerate(args) if isinstance(elem, torch.Tensor)]
+
+        # only if gradient required
+        if not any(tensor.requires_grad for tensor in tensors):
+            return None, args, tensor_mask
+
+        # add identity to ensure .grad_fn exists and all tensors share the same .grad_fn
+        post_tensors = Identity.apply(*tensors)
+        grad_fn = next((tensor.grad_fn for tensor in post_tensors if tensor.grad_fn is not None), None)
+        if grad_fn is None:
+            # sanity check, should never happen because the check above already catches cases in which no input tensor
+            # requires a gradient, and in normal conditions, we will always obtain a grad_fn from `Identity` for each
+            # tensor with requires_grad=True
+            raise RuntimeError('Backward hook could not be registered!')  # pragma: no cover
+
+        # work-around to support in-place operations
+        post_tensors = tuple(elem.clone() for elem in post_tensors)
+        post_args = tuple(uncompress(args, tensor_mask, post_tensors))
+        return grad_fn, post_args, tensor_mask
+
+    def pre_forward(self, module, args, kwargs):
+        '''Apply an Identity to the input before the module to register a backward hook.
+
+        Parameters
+        ----------
+        module: :py:obj:`torch.nn.Module`
+            The module to which this hook is attached.
+        args: tuple of :py:obj:`torch.Tensor`
+            The input tensors passed to ``module.forward``.
+        kwargs: dict
+            The keyword arguments passed to ``module.forward``.
+
+        Returns
+        -------
+        tuple of :py:obj:`torch.Tensor`, optional
+            A tuple of the modified input tensors.
+
+        '''
         hook_ref = weakref.ref(self)
 
+        grad_fn, post_args, input_tensor_mask = self._inject_grad_fn(args)
+        if grad_fn is None:
+            return None
+
         @functools.wraps(self.backward)
         def wrapper(grad_input, grad_output):
             hook = hook_ref()
             if hook is not None and hook.active:
-                return hook.backward(module, grad_input, hook.stored_tensors['grad_output'])
+                return hook.backward(
+                    module,
+                    list(uncompress(
+                        repeat(None),
+                        input_tensor_mask,
+                        grad_input,
+                    )),
+                    hook.stored_tensors['grad_output'],
+                )
             return None
 
-        if not isinstance(input, tuple):
-            input = (input,)
+        # register the input tensor gradient hook
+        self.tensor_handles.append(grad_fn.register_hook(wrapper))
 
-        # only if gradient required
-        if input[0].requires_grad:
-            # add identity to ensure .grad_fn exists
-            post_input = Identity.apply(*input)
-            # register the input tensor gradient hook
-            self.tensor_handles.append(
-                post_input[0].grad_fn.register_hook(wrapper)
-            )
-            # work around to support in-place operations
-            post_input = tuple(elem.clone() for elem in post_input)
-        else:
-            # no gradient required
-            post_input = input
-        return post_input[0] if len(post_input) == 1 else post_input
+        return post_args, kwargs
 
-    def post_forward(self, module, input, output):
-        '''Register a backward-hook to the resulting tensor right after the forward.'''
+    def post_forward(self, module, args, kwargs, output):
+        '''Register a backward-hook to the resulting tensor right after the forward.
+
+        Parameters
+        ----------
+        module: :py:obj:`torch.nn.Module`
+            The module to which this hook is attached.
+        args: tuple of :py:obj:`torch.Tensor`
+            The input tensors passed to ``module.forward``.
+        kwargs: tuple of object
+            The keyword arguments passed to ``module.forward``.
+        output: :py:obj:`torch.Tensor`
+            The output tensor.
+
+        Returns
+        -------
+        tuple of :py:obj:`torch.Tensor`, optional
+            A tuple of the modified output tensors.
+        '''
         hook_ref = weakref.ref(self)
 
+        single = not isinstance(output, tuple)
+        if single:
+            output = (output,)
+
+        grad_fn, post_output, output_tensor_mask = self._inject_grad_fn(output)
+        if grad_fn is None:
+            return None
+
         @functools.wraps(self.pre_backward)
         def wrapper(grad_input, grad_output):
             hook = hook_ref()
             if hook is not None and hook.active:
-                return hook.pre_backward(module, grad_input, grad_output)
+                return hook.pre_backward(
+                    module,
+                    grad_input,
+                    tuple(uncompress(
+                        repeat(None),
+                        output_tensor_mask,
+                        grad_output
+                    ))
+                )
             return None
 
-        if not isinstance(output, tuple):
-            output = (output,)
+        # register the output tensor gradient hook
+        self.tensor_handles.append(grad_fn.register_hook(wrapper))
 
-        # only if gradient required
-        if output[0].grad_fn is not None:
-            # register the output tensor gradient hook
-            self.tensor_handles.append(
-                output[0].grad_fn.register_hook(wrapper)
-            )
-        return output[0] if len(output) == 1 else output
+        if single:
+            return post_output[0]
+        return post_output
 
     def pre_backward(self, module, grad_input, grad_output):
-        '''Store the grad_output for the backward hook'''
+        '''Store the grad_output for the backward hook.
+
+        Parameters
+        ----------
+        module: :py:obj:`torch.nn.Module`
+            The module to which this hook is attached.
+        grad_input: :py:obj:`torch.Tensor`
+            The input gradient tensor.
+        grad_output: :py:obj:`torch.Tensor`
+            The output gradient tensor.
+        '''
         self.stored_tensors['grad_output'] = grad_output
 
-    def forward(self, module, input, output):
-        '''Hook applied during forward-pass'''
+    def forward(self, module, args, kwargs, output):
+        '''Hook applied during forward-pass.
+
+        Parameters
+        ----------
+        module: :py:obj:`torch.nn.Module`
+            The module to which this hook is attached.
+        args: tuple of :py:obj:`torch.Tensor`
+            The input tensors passed to ``module.forward``.
+        kwargs: tuple of object
+            The keyword arguments passed to ``module.forward``.
+        output: :py:obj:`torch.Tensor`
+            The output tensor.
+        '''
 
     def backward(self, module, grad_input, grad_output):
-        '''Hook applied during backward-pass'''
+        '''Hook applied during backward-pass.
+
+        Parameters
+        ----------
+        module: :py:obj:`torch.nn.Module`
+            The module to which this hook is attached.
+        grad_input: :py:obj:`torch.Tensor`
+            The input gradient tensor.
+        grad_output: :py:obj:`torch.Tensor`
+            The output gradient tensor.
+        '''
 
     def copy(self):
         '''Return a copy of this hook.
         This is used to describe hooks of different modules by a single hook instance.
+
+        Returns
+        -------
+        :py:obj:`BasicHook`
+            A copy of this hook.
+
         '''
         return self.__class__()
 
     def remove(self):
-        '''When removing hooks, remove all references to stored tensors'''
+        '''When removing hooks, remove all references to stored tensors.'''
         self.stored_tensors.clear()
         self.tensor_handles.remove()
 
     def register(self, module):
-        '''Register this instance by registering all hooks to the supplied module.'''
+        '''Register this instance by registering all hooks to the supplied module.
+
+        Parameters
+        ----------
+        module: :py:obj:`torch.nn.Module`
+            The module to which to register to.
+
+        Returns
+        -------
+        :py:obj:`RemovableHandleList`
+            A list of removable handles, one for each registered hook.
+
+        '''
+        def with_kwargs(method, has_output=True):
+            '''Check whether the method uses args/kwargs, or only inputs. This ensures compatibility with rules that do
+            not consider kwargs, and reduces code clutter.
+
+            Parameters
+            ----------
+            method: function
+                Function to check.
+            has_output: bool
+                Function to check.
+
+            Returns
+            -------
+            bool
+                True if `method` uses kwargs.
+            '''
+            params = signature(method).parameters
+            # assume with_kwargs if forward has not 3 parameters and 3rd is not called 'output'
+            if has_output:
+                return len(params) != 3 and list(params)[2] != 'output'
+            # e.g., pre_forward has no output, so we expect 2 parameters
+            return len(params) != 2
+
         return RemovableHandleList([
             RemovableHandle(self),
-            module.register_forward_pre_hook(self.pre_forward),
-            module.register_forward_hook(self.post_forward),
-            module.register_forward_hook(self.forward),
+            module.register_forward_pre_hook(self.pre_forward, with_kwargs=with_kwargs(self.pre_forward, False)),
+            module.register_forward_hook(self.post_forward, with_kwargs=with_kwargs(self.post_forward)),
+            module.register_forward_hook(self.forward, with_kwargs=with_kwargs(self.forward)),
         ])
 
 
@@ -487,6 +715,10 @@ class BasicHook(Hook):
         inputs and gradients produced through ``input_modifiers`` and ``param_modifiers``. ``inputs`` and ``gradients``
         have the same as ``input_modifiers`` and ``param_modifiers``. Default is the sum of the multiplications of each
         input and its corresponding gradient.
+    stabilizer: callable or float, optional
+        Stabilization parameter for rules other than ``Epsilon``. If ``stabilizer`` is a float, it will be added to the
+        denominator with the same sign as each respective entry. If it is callable, a function ``(input: torch.Tensor)
+        -> torch.Tensor`` is expected, of which the output corresponds to the stabilized denominator.
     '''
     def __init__(
         self,
@@ -518,19 +750,49 @@ class BasicHook(Hook):
         self.gradient_mapper = gradient_mapper
         self.reducer = reducer
 
-    def forward(self, module, input, output):
-        '''Forward hook to save module in-/outputs.'''
-        self.stored_tensors['input'] = input
+    def forward(self, module, args, kwargs, output):
+        '''Forward hook to save module in-/outputs.
+
+        Parameters
+        ----------
+        module: :py:obj:`torch.nn.Module`
+            The module to which this hook is attached.
+        args: tuple of :py:obj:`torch.Tensor`
+            The input tensors passed to ``module.forward``.
+        kwargs: tuple of object
+            The keyword arguments passed to ``module.forward``.
+        output: :py:obj:`torch.Tensor`
+            The output tensor.
+        '''
+        self.stored_tensors['input'] = args
+        self.stored_tensors['kwargs'] = kwargs
 
     def backward(self, module, grad_input, grad_output):
-        '''Backward hook to compute LRP based on the class attributes.'''
-        original_input = self.stored_tensors['input'][0].clone()
+        '''Backward hook to compute LRP based on the class attributes.
+
+        Parameters
+        ----------
+        module: :py:obj:`torch.nn.Module`
+            The module to which this hook is attached.
+        grad_input: :py:obj:`torch.Tensor`
+            The input gradient tensor.
+        grad_output: :py:obj:`torch.Tensor`
+            The output gradient tensor.
+
+        Returns
+        -------
+        tuple of :py:obj:`torch.nn.Module`
+            The modified input gradient tensors.
+        '''
+        original_input, *original_args = self.stored_tensors['input']
+        original_input = original_input.clone()
+        original_kwargs = self.stored_tensors['kwargs']
         inputs = []
         outputs = []
         for in_mod, param_mod, out_mod in zip(self.input_modifiers, self.param_modifiers, self.output_modifiers):
             input = in_mod(original_input).requires_grad_()
             with ParamMod.ensure(param_mod)(module) as modified, torch.autograd.enable_grad():
-                output = modified.forward(input)
+                output = modified.forward(input, *original_args, **original_kwargs)
                 output = out_mod(output)
             inputs.append(input)
             outputs.append(output)
@@ -547,6 +809,11 @@ class BasicHook(Hook):
     def copy(self):
         '''Return a copy of this hook.
         This is used to describe hooks of different modules by a single hook instance.
+
+        Returns
+        -------
+        :py:obj:`BasicHook`
+            A copy of this hook.
         '''
         copy = BasicHook.__new__(type(self))
         BasicHook.__init__(
@@ -573,7 +840,13 @@ class BasicHook(Hook):
 
 
 class RemovableHandle:
-    '''Create weak reference to call .remove on some instance.'''
+    '''Create weak reference to call .remove on some instance.
+
+    Parameters
+    ----------
+    instance: object
+        The instance to which to create the reference.
+    '''
     def __init__(self, instance):
         self.instance_ref = weakref.ref(instance)
 
@@ -689,9 +962,14 @@ class Composite:
         return CompositeContext(module, self)
 
     @contextmanager
-    def inactive(self):
+    def inactive(self) -> Generator:
         '''Context manager to temporarily deactivate the gradient modification. This can be used to compute the
         gradient of the modified gradient.
+
+        Yields
+        ------
+        self: :py:obj:`Composite`
+            The instance of this composite.
         '''
         try:
             for hook in self.hook_refs:
@@ -703,5 +981,20 @@ class Composite:
 
     @staticmethod
     def _empty_module_map(ctx, name, module):
-        '''Empty module_map, does not assign any rules.'''
+        '''Empty module_map, does not assign any rules.
+
+        Parameters
+        ----------
+        ctx: dict
+            A dictionary containing an optional context.
+        name: str
+            The name of the module.
+        module: :py:obj:`torch.nn.Module`
+            The instance of the module.
+
+        Returns
+        -------
+        NoneType
+            Always `None`, as the empty module map will never register any hooks.
+        '''
         return None
