@@ -17,16 +17,39 @@
 # along with this library. If not, see <https://www.gnu.org/licenses/>.
 '''Type definitions for convenience.'''
 import torch
+import torchvision
 
 
 class SubclassMeta(type):
     '''Meta class to bundle multiple subclasses.'''
     def __instancecheck__(cls, inst):
-        """Implement isinstance(inst, cls) as subclasscheck."""
+        """Implement isinstance(inst, cls) as subclasscheck.
+
+        Parameters
+        ----------
+        inst: object
+            The instance to check.
+
+        Returns
+        -------
+        bool
+            True, if `inst` is an instance of `cls`.
+        """
         return cls.__subclasscheck__(type(inst))
 
     def __subclasscheck__(cls, sub):
-        """Implement issubclass(sub, cls) with by considering additional __subclass__ members."""
+        """Implement issubclass(sub, cls) with by considering additional __subclass__ members.
+
+        Parameters
+        ----------
+        sub: object
+            The type to check.
+
+        Returns
+        -------
+        bool
+            True, if `sub` is an instance of `cls`.
+        """
         candidates = cls.__dict__.get("__subclass__", tuple())
         return type.__subclasscheck__(cls, sub) or issubclass(sub, candidates)
 
@@ -71,6 +94,7 @@ class BatchNorm(metaclass=SubclassMeta):
         torch.nn.modules.batchnorm.BatchNorm1d,
         torch.nn.modules.batchnorm.BatchNorm2d,
         torch.nn.modules.batchnorm.BatchNorm3d,
+        torchvision.ops.FrozenBatchNorm2d,
     )
 
 
@@ -108,7 +132,6 @@ class Activation(metaclass=SubclassMeta):
         torch.nn.modules.activation.Hardswish,
         torch.nn.modules.activation.LeakyReLU,
         torch.nn.modules.activation.LogSigmoid,
-        torch.nn.modules.activation.MultiheadAttention,
         torch.nn.modules.activation.PReLU,
         torch.nn.modules.activation.ReLU,
         torch.nn.modules.activation.ReLU6,
